import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;

public class PicturePerfect extends JApplet
                            implements ActionListener
{
   private static final int NB_COORDINATES = 1024;
   private static final Color [] COLOR_SET =
   {
      Color.red, Color.green, Color.blue
   };
   private static final String [] COLOR_NAMES =
   {
      "Red", "Green", "Blue"
   };

   private int NbCoordinates,
               CurCoordinate;
   private int [] _x;
   private int [] _y;
   private Color [] _color;

   // -------------
   // Current color (in use for next line)
   // -------------
   private Color curColor;
   // -------------
   // Accessors for the current color
   // -------------
   private void SetCurColor (Color c)
   {
      curColor = c;
   }
   private Color GetCurColor ()
   {
      return curColor;
   }


   private JCheckBoxMenuItem mnuPenDown;
   private JRadioButtonMenuItem [] mnuColor;
   private JPopupMenu mainMenu;


   // -------------
   // Current pen status (if down, the user can draw)
   // -------------
   private boolean penDown;
   // -------------
   // Accessors for the pen status (setting the pen
   // updates the status bar)
   // -------------
   private void SetPen (boolean Down)
   {
      penDown = Down;
      mnuPenDown.setState (Down);
      UpdateStatus ();
   }
   private boolean GetPen ()
   {
      return penDown;
   }


   // -------------
   // Internal class DrawingPanel, to draw (of course).
   // It uses other instance members of my applet.
   // -------------
   private class DrawingPanel extends JPanel
   {
      public DrawingPanel ()
      {
         super ();
         setBackground (Color.black);
      }

      public void paint (Graphics g)
      {
         super.paint (g);
         Graphics2D g2D = (Graphics2D) g;
         if (NbCoordinates > 0)
         {
            int oldX = _x[0],
                oldY = _y[0];

            for (int i = 1; i < NbCoordinates; i++)
            {
               g2D.setColor (_color[i]);
               g2D.draw (new Line2D.Float ( (float)oldX,
                                            (float)oldY,
                                            (float)_x[i],
                                            (float)_y[i]));
               oldX = _x[i];
               oldY = _y[i];
            }
         }
      }
   }
   // -------------
   // The DrawingPanel of my applet
   // -------------
   private DrawingPanel p;

   // -------------
   // The status bar of my applet
   // -------------
   private JLabel statusLabel;
   // -------------
   // Method to update the text on the status bar
   // -------------
   private void UpdateStatus ()
   {
      if (GetPen ())
         statusLabel.setText ("Pen is down; you can draw. Use contextual menu for options");
      else
         statusLabel.setText ("Pen is up; you cannot draw. Use contextual menu for options");
   }


   private void AddCoordinates (int x, int y)
   {
      _x[CurCoordinate] = x;
      _y[CurCoordinate] = y;
      _color[CurCoordinate] = curColor;
      CurCoordinate = (CurCoordinate + 1) % NB_COORDINATES;
      if (NbCoordinates < NB_COORDINATES)
         NbCoordinates++;
   }


   public PicturePerfect ()
   {
      super ();
      _x = new int [NB_COORDINATES];
      _y = new int [NB_COORDINATES];
      _color = new Color [NB_COORDINATES];
      NbCoordinates = 0; // so far
      CurCoordinate = 0; // so far
      SetCurColor (COLOR_SET[0]);
   }

   public void init ()
   {
      boolean initialPenStatus = false;

      statusLabel = new JLabel ("");
      getContentPane ().add (statusLabel, BorderLayout.SOUTH);
      p = new DrawingPanel ();
      getContentPane ().add (p, BorderLayout.CENTER);
      p.setBackground (Color.black);

      mainMenu = new JPopupMenu ("Options");

      mnuPenDown = new JCheckBoxMenuItem ("Pen down", initialPenStatus);
      mainMenu.add (mnuPenDown);
      mnuPenDown.addActionListener (this);

      SetPen (initialPenStatus);

      mainMenu.addSeparator ();

      ButtonGroup colorGroup = new ButtonGroup ();

      mnuColor = new JRadioButtonMenuItem [COLOR_SET.length];
      for (int i = 0; i < mnuColor.length; i++)
      {
         // Only the zero-th item is set initially
         mnuColor[i] = new JRadioButtonMenuItem (COLOR_NAMES[i], (i == 0));
         mainMenu.add (mnuColor[i]);
         mnuColor[i].addActionListener (this);
         colorGroup.add (mnuColor[i]);
      }

      addMouseMotionListener (
         new MouseMotionAdapter ()
         {
            public void mouseDragged (MouseEvent e)
            {
               if (GetPen ())
               {
                  AddCoordinates (e.getX (), e.getY ());
                  repaint ();
               }
            }
         }
      );

      addMouseListener (
         new MouseAdapter ()
         {
            public void mousePressed (MouseEvent e)
            {
               ShowPopup (e);
            }
            public void mouseReleased (MouseEvent e)
            {
               ShowPopup (e);
            }
            private boolean ShowPopup (MouseEvent e)
            {
               boolean IDidIt = false;
               if (e.isPopupTrigger ())
               {
                  mainMenu.show (e.getComponent (),
                                 e.getX (), e.getY ());
                  IDidIt = true;
               }
               return IDidIt;

            }
         }
      );
   }

   public void actionPerformed (ActionEvent e)
   {
      if (e.getSource () == mnuPenDown)
      {
         SetPen (!GetPen ());
      }
      else // It's a color change
      {
         for (int i = 0; i < mnuColor.length; i++)
         {
            if (e.getSource () == mnuColor[i])
            {
               SetCurColor (COLOR_SET[i]);
            }
         }
      }
   }
}
