import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; // ListSelectionListener, ListSelectionEvent, etc.

public class LookAndFeelExplorer extends JFrame
                                 implements ListSelectionListener
{
   private String [] LookAndFeelNames;
   private UIManager.LookAndFeelInfo [] Aspects;
   private JList AspectList;

   private void LoadLookandFeelNames ()
   {
      Aspects = UIManager.getInstalledLookAndFeels ();
      if (Aspects.length > 0)
      {
         LookAndFeelNames = new String [Aspects.length];
         for (int i = 0; i < LookAndFeelNames.length; i++)
            LookAndFeelNames[i] = new String (Aspects[i].getClassName ());
      }
      else
      {
         LookAndFeelNames = null;
      }
   }

   LookAndFeelExplorer ()
   {
      super ();
      LoadLookandFeelNames ();
      init ();
   }

   LookAndFeelExplorer (String Title)
   {
      super (Title);
      LoadLookandFeelNames ();
      init ();
   }


   public void init ()
   {
      final String DESCRIPTIVE_TEXT = "Selected look and feel";

      JPanel HeadPane = new JPanel ();
      HeadPane.setLayout (new FlowLayout ());
      HeadPane.add (new JLabel (DESCRIPTIVE_TEXT));

      // Voir 584-587 D&D
      if (LookAndFeelNames != null)
      {
         AspectList = new JList (LookAndFeelNames);
         AspectList.setSelectionMode (ListSelectionModel.SINGLE_INTERVAL_SELECTION);
         AspectList.setVisibleRowCount (LookAndFeelNames.length);
         AspectList.setSelectedIndex (0);
      }
      else
      {
         String [] temp =  { "None" };
         AspectList = new JList (temp);
         AspectList.setSelectionMode (ListSelectionModel.SINGLE_INTERVAL_SELECTION);
         AspectList.setEnabled (false);
      }

      AspectList.addListSelectionListener (this);
      HeadPane.add (new JScrollPane (AspectList));

      getContentPane ().add (HeadPane, BorderLayout.NORTH);


      // -- StuffPanel
      final String [] EXAMPLE_LIST= { "Element 1", "Element 2", "Element 3" };
      JPanel StuffPanel = new JPanel ();
      StuffPanel.setLayout (new GridLayout (2, 3));

      JRadioButton [] jrb = new JRadioButton [3];
      ButtonGroup jbg = new ButtonGroup ();
      for (int i = 0; i < jrb.length; i++)
      {
         jrb[i] = new JRadioButton ("Example RadioButton " + i);
         StuffPanel.add (jrb[i]);
         jbg.add (jrb[i]);
      }

      jrb[0].setSelected (true);

      StuffPanel.add (new JCheckBox("Example CheckBox"));
      StuffPanel.add (new JButton ("Example Button"));
      StuffPanel.add (new JComboBox (EXAMPLE_LIST));

      getContentPane ().add (StuffPanel, BorderLayout.CENTER);

      addWindowListener (
         new WindowAdapter ()
         {
            public void windowClosing (WindowEvent e)
            {
               System.exit (0);
            }
         }
      );
   }

   public void valueChanged (ListSelectionEvent ev)
   {
      int Selection = AspectList.getSelectedIndex ();
      try
      {
         UIManager.setLookAndFeel (Aspects[Selection].getClassName ()); // INVESTIGATE
         SwingUtilities.updateComponentTreeUI (this);
      }
      catch (Exception e)
      {
        e.printStackTrace (); // INVESTIGATE
      }
   }


   public static void main (String [] args)
   {
      LookAndFeelExplorer frame = new LookAndFeelExplorer ("Look and Feel Explorer");

      frame.setVisible (true);
      frame.pack ();
   }
}


