import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class MonthPicker extends JFrame implements ActionListener, ItemListener
{
   // --- Month-related attributes and methods
   public static final int
      JAN = 0, FEB =  1, MAR =  2,
      APR = 3, MAY =  4, JUN =  5,
      JUL = 6, AUG =  7, SEP =  8,
      OCT = 9, NOV = 10, DEC = 11,
      NB_MONTHS = 12;

   public final String [] MONTHS =
   {
      "January", "February", "March",
      "April",   "May",      "June",
      "July",    "August",   "September",
      "October", "November", "December"
   };

   private int CurrentMonth;

   private JLabel CurrentMonthName;
   private JTextArea OnThisMonth;
   private JLabel AppStatus;
   private JButton NextMonthButton,
                   PrevMonthButton;
   private JComboBox MonthSelector;


   public static boolean isValidMonth (int month)
   {
      return month >= JAN && month <= DEC;
   }

   // No validation; for internal use only
   private void InternalSetCurrentMonth (int curMonth)
   {
      CurrentMonth = curMonth;
   }

   // Validation; for common usage
   public void SetCurrentMonth (int curMonth) throws Exception
   {
      if (!isValidMonth (curMonth))
         throw new Exception ("Month out of bounds " + curMonth);
      InternalSetCurrentMonth (CurrentMonth);
   }

   public int GetCurrentMonth ()
   {
      return CurrentMonth;
   }

   public void NextMonth ()
   {
      InternalSetCurrentMonth ((GetCurrentMonth () + 1) % NB_MONTHS);
   }

   public void PreviousMonth ()
   {
      if (GetCurrentMonth () == JAN)
         InternalSetCurrentMonth (DEC);
      else
         InternalSetCurrentMonth (GetCurrentMonth () - 1);
   }



   public MonthPicker ()
   {
      super ();
      InternalSetCurrentMonth (JAN);
      init ();
   }

   public MonthPicker (String Title)
   {
      super (Title);
      InternalSetCurrentMonth (JAN);
      init ();
   }


   public void init ()
   {
      addWindowListener (new WindowAdapter() { public void windowClosing (WindowEvent e) { System.exit (0); } } );

      JPanel ControlPanel = new JPanel (),
             DataPanel    = new JPanel (),
             StatusPanel  = new JPanel ();

      // Fill ControlPanel
      MonthSelector = new JComboBox (MONTHS);
      NextMonthButton = new JButton ("Next Month -->");
      PrevMonthButton = new JButton ("<-- Prev Month");

      NextMonthButton.addActionListener (this);
      PrevMonthButton.addActionListener (this);
      MonthSelector.addItemListener     (this);

      ControlPanel.setLayout (new GridLayout (6, 1));
      Box b = Box.createHorizontalBox ();
      b.add (PrevMonthButton);
      b.add (NextMonthButton);

      // The first four rows are empty
      ControlPanel.add (new JLabel (""));
      ControlPanel.add (new JLabel (""));
      ControlPanel.add (new JLabel (""));
      ControlPanel.add (new JLabel (""));
      ControlPanel.add (b);
      ControlPanel.add (MonthSelector);

      // Fill DataPanel
      final int ON_THIS_MONTH_WIDTH  = 20,
                ON_THIS_MONTH_HEIGHT = 10;

      DataPanel.setLayout (new BorderLayout ());

      OnThisMonth = new JTextArea ("", ON_THIS_MONTH_HEIGHT,
                                       ON_THIS_MONTH_WIDTH);
      CurrentMonthName = new JLabel ("");
      DataPanel.add (CurrentMonthName, BorderLayout.NORTH);
      DataPanel.add (OnThisMonth, BorderLayout.CENTER);

      // Fill StatusPanel
      StatusPanel.setLayout (new FlowLayout ());
      AppStatus = new JLabel ("Please pick a month");
      StatusPanel.add (AppStatus);

      getContentPane ().add (ControlPanel, BorderLayout.WEST);
      getContentPane ().add (DataPanel,    BorderLayout.EAST);
      getContentPane ().add (StatusPanel,  BorderLayout.SOUTH);

      UpdateMonthDisplay ();
   }

   private void UpdateMonthDisplay ()
   {
      CurrentMonthName.setText ("To do on the month of " +
                                MONTHS[GetCurrentMonth ()]);
      if (GetCurrentMonth () != MonthSelector.getSelectedIndex ())
         MonthSelector.setSelectedIndex (GetCurrentMonth ());
         
      pack ();
   }


   public void actionPerformed (ActionEvent e)
   {
      if (e.getSource () == NextMonthButton)
         NextMonth ();
      else if (e.getSource () == PrevMonthButton)
         PreviousMonth ();

      UpdateMonthDisplay();
   }

   public void itemStateChanged (ItemEvent e)
   {
      // We know it's MonthSelector (for now)
      InternalSetCurrentMonth (MonthSelector.getSelectedIndex ());
      UpdateMonthDisplay ();
   }

   public static void main (String [] args)
   {
      MonthPicker frame = new MonthPicker ("Pick a month!");
      frame.pack ();
      frame.setVisible (true);
   }
}
